﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using PIbd_24_EredavkinRA_BusBusiness.Entities;


namespace PIbd_24_EredavkinRA_BusBusiness.Repositories.Implementations;

public class RoutSheetRepository : IRoutSheetRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<RoutSheetRepository> _logger;

    public RoutSheetRepository(IConnectionString connectionString, ILogger<RoutSheetRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateRoutSheet(RoutSheet routsheet)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(routsheet));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO RoutSheet (RoutId, DriverId, ConductorId, BusDate)
                VALUES (@RoutId, @DriverId, @ConductorID, @BusDate);
                SELECT MAX(Id) FROM RoutSheet";
            var routsheetId = connection.QueryFirst<int>(queryInsert,  routsheet, transaction);

            var querySubInsert = @"
                INSERT INTO RoutSheetBus (RoutSheetId, BusId)
                VALUES (@RoutSheetId, @BusId)";
            foreach (var elem in routsheet.RoutSheetBus)
            {
                connection.Execute(querySubInsert, new { routsheetId, elem.BusID }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

   

    public IEnumerable<RoutSheet> ReadRoutSheet(DateTime? dateForm = null, DateTime? dateBus = null, int? routId = null, int? busid = null, int? driverid = null, int? conductorid = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM RoutSheet";
            var routsheet = connection.Query<RoutSheet>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(routsheet));
            return routsheet;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
